/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2011-2017, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    ArmarX
 * @author     Mirko Waechter( mirko.waechter at kit dot edu)
 * @date       2018
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include <chrono>
#include "ControlThreadOutputBuffer.h"

namespace armarx
{
    namespace rt_timing::constants
    {
        inline constexpr const std::int64_t seconds2MicroSeconds = 1e6;

        static constexpr const std::int64_t nanoSeconds2MicroSeconds = 1000;
    } // namespace rt_timing::constants

    inline IceUtil::Time
    rtNow(clockid_t clockId = CLOCK_MONOTONIC_RAW)
    {
        using namespace rt_timing::constants;
        struct timespec ts;
        clock_gettime(clockId, &ts);
        return IceUtil::Time::microSeconds(ts.tv_sec * seconds2MicroSeconds +
                                           ts.tv_nsec / nanoSeconds2MicroSeconds);
    }
} // namespace armarx

#define RT_TIMING_START(name) auto name = armarx::rtNow();
//! \ingroup VirtualTime
//! Prints duration with comment in front of it, yet only once per second.
#define RT_TIMING_END_COMMENT(name, comment)                                                       \
    printf(                                                                           \
        "%s - duration: %.3f ms \n", comment, (armarx::rtNow() - name).toMilliSecondsDouble());
//! \ingroup VirtualTime
//! Prints duration
#define RT_TIMING_END(name) RT_TIMING_END_COMMENT(name, #name)
//! \ingroup VirtualTime
//! Prints duration with comment in front of it if it took longer than threshold
#define RT_TIMING_CEND_COMMENT(name, comment, thresholdMs)                                         \
    if ((armarx::rtNow() - name).toMilliSecondsDouble() >= thresholdMs)                            \
    RT_TIMING_END_COMMENT(name, comment)
//! \ingroup VirtualTime
//! Prints duration if it took longer than thresholdMs
#define RT_TIMING_CEND(name, thresholdMs) RT_TIMING_CEND_COMMENT(name, #name, thresholdMs)
