#include "Processors.h"

#include <ArmarXCore/core/logging/Logging.h>

#include "converter/data/image/exr/ExrConverter.h"
#include "converter/data/image/png/PngConverter.h"
#include "extractor/imageExtractor/DepthImageExtractor.h"
#include "extractor/imageExtractor/ImageExtractor.h"
#include "filter/equalityFilter/EqualityFilter.h"
#include "filter/frequencyFilter/FrequencyFilter.h"

namespace armarx::armem::server::ltm
{
    void
    Processors::configure(const nlohmann::json& config)
    {
        // Filters:
        if (config.contains(processor::filter::SnapshotFrequencyFilter::NAME))
        {
            ARMARX_IMPORTANT << "ADDING SNAPSHOT FILTER";
            auto f = std::make_unique<processor::filter::SnapshotFrequencyFilter>();
            f->configure(config[processor::filter::SnapshotFrequencyFilter::NAME]);
            snapFilters.push_back(std::move(f));
        }

        // Converters
        if (config.contains(processor::converter::data::image::PngConverter::NAME))
        {
            ARMARX_IMPORTANT << "ADDING IMG CONVERTER";
            auto f = std::make_unique<processor::converter::data::image::PngConverter>();
            f->configure(config[processor::converter::data::image::PngConverter::NAME]);
            converters.push_back(std::move(f));
        }
    }
} // namespace armarx::armem::server::ltm
