#include "PngConverter.h"

// ArmarX
#include <opencv2/imgcodecs.hpp>
#include <opencv2/imgproc.hpp>
#include <opencv2/opencv.hpp>

#include <RobotAPI/libraries/aron/converter/opencv/OpenCVConverter.h>

namespace armarx::armem::server::ltm::processor::converter::data::image
{
    void
    PngConverter::configure(const nlohmann::json& json)
    {
    }

    PngConverter::ConversionResult
    PngConverter::_convert(const aron::data::NDArrayPtr& data)
    {
        ARMARX_CHECK_NOT_NULL(data);

        auto img = aron::data::converter::AronOpenCVConverter::ConvertToMat(data);
        std::vector<unsigned char> buffer;


        auto shape = data->getShape(); // we know from the extraction that the shape has 3 elements
        ARMARX_CHECK_EQUAL(shape.size(), 3);

        if (shape[2] == 3) // its probably a rgb image
        {
            cv::cvtColor(img, img, CV_RGB2BGR);
            cv::imencode(suffix, img, buffer);
            return {buffer, ".rgb"};
        }

        if (shape[2] == 1) // its probably a grayscale image
        {
            cv::imencode(suffix, img, buffer);
            return {buffer, ".gs"};
        }

        // try to export without conversion
        cv::imencode(suffix, img, buffer);
        return {buffer, ""};
    }

    aron::data::NDArrayPtr
    PngConverter::_convert(const ConversionResult& data, const armarx::aron::Path& p)
    {
        if (data.suffix == ".rgb")
        {
            cv::Mat img = cv::imdecode(data.data, cv::IMREAD_COLOR);
            cv::cvtColor(img, img, CV_BGR2RGB);
            return aron::data::converter::AronOpenCVConverter::ConvertFromMat(img, p);
        }

        if (data.suffix == ".gs")
        {
            cv::Mat img = cv::imdecode(data.data, cv::IMREAD_GRAYSCALE);
            return aron::data::converter::AronOpenCVConverter::ConvertFromMat(img, p);
        }

        // try to load without conversion
        cv::Mat img = cv::imdecode(data.data, cv::IMREAD_ANYCOLOR);
        return aron::data::converter::AronOpenCVConverter::ConvertFromMat(img, p);
    }
} // namespace armarx::armem::server::ltm::processor::converter::data::image
