/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    robdekon
 * @author     Christoph Pohl ( christoph dot pohl at kit dot edu )
 * @date       07.09.22
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <experimental/memory>

#include <ArmarXCore/core/ComponentPlugin.h>
#include <ArmarXCore/core/ManagedIceObject.h>

#include <ArmarXGui/libraries/ArmarXGuiComponentPlugins/RemoteGuiComponentPlugin.h>

#include <RobotAPI/libraries/aron/core/codegeneration/cpp/AronGeneratedClass.h>
#include <RobotAPI/libraries/aron/core/data/visitor/RecursiveVisitor.h>

#include "PropertyDefinitionVisitors.h"
#include "RemoteGui.h"

namespace armarx::plugins
{
    template <typename T>
    concept isAronGenerated = std::is_base_of<armarx::aron::cpp::AronGeneratedClass, T>::value;

    template <typename AronStructT>
        requires isAronGenerated<AronStructT>
    class AronComponentConfigPlugin : public ComponentPlugin
    {
    protected:
        void
        preOnInitComponent() override
        {
            ARMARX_TRACE;
            armarx::aron::component_config::PropertyDefinitionGetterVisitor vis(
                parent<armarx::PropertyUser>(), prefix());
            auto data = std::static_pointer_cast<armarx::aron::data::Variant>(
                config_.getUpToDateReadBuffer().toAron());
            auto type = config_.getUpToDateReadBuffer().ToAronType();
            armarx::aron::data::visitRecursive(vis, data, type);
            config_.getWriteBuffer().fromAron(
                std::static_pointer_cast<armarx::aron::data::Dict>(data));
            config_.commitWrite();
            ManagedIceObjectPlugin::preOnInitComponent();
        }

        void
        postCreatePropertyDefinitions(armarx::PropertyDefinitionsPtr& properties) override
        {
            ARMARX_TRACE;
            armarx::aron::component_config::PropertyDefinitionSetterVisitor vis(properties, prefix());
            const auto& config = config_.getUpToDateReadBuffer();
            armarx::aron::data::visitRecursive(vis, config.toAron(), config.ToAronType());
            ComponentPlugin::postCreatePropertyDefinitions(properties);
        }

    public:
        RemoteGui::detail::GroupBoxBuilder
        buildRemoteGui(const std::string& name)
        {
            ARMARX_TRACE;
            return RemoteGui::makeConfigGui(name, config_.getUpToDateReadBuffer());
        }

        bool
        updateRemoteGui(armarx::RemoteGui::TabProxy&
                            prx) // prx has to be already updated; otherwise nothing will change
        {
            ARMARX_TRACE;
            armarx::aron::component_config::GetValueFromMapVisitor vis(&prx);
            auto& cfg = config_.getWriteBuffer();
            auto data = std::static_pointer_cast<armarx::aron::data::Variant>(cfg.toAron());
            auto type = cfg.ToAronType();
            armarx::aron::data::visitRecursive(vis, data, type);
            cfg.fromAron(std::static_pointer_cast<armarx::aron::data::Dict>(data));
            config_.commitWrite();
            return vis.tabRebuildRequired();
        }

        AronComponentConfigPlugin(ManagedIceObject& parent, const std::string& prefix) :
            ComponentPlugin(parent, prefix)
        {
            ARMARX_TRACE;
        }

        armarx::WriteBufferedTripleBuffer<AronStructT> config_;
    };
} // namespace armarx::plugins