#pragma once

#include <map>
#include <memory>
#include <string>
#include <vector>

#include <RobotAPI/interface/robot_name_service/RobotNameServiceInterface.h>
#include <RobotAPI/libraries/armem/client/MemoryNameSystem.h>

namespace armarx::robot_name_service::core
{
    class TCPInfo
    {
    public:
        void fromIce(const armarx::robot_name_service::dto::TCPInfo& r);
        armarx::robot_name_service::dto::TCPInfo toIce() const;

    public:
        std::string name;
        std::string ft_name;
        armarx::HandUnitInterfacePrx handUnitInterface;
    };

    class ArmInfo
    {
    public:
        void fromIce(const armarx::robot_name_service::dto::ArmInfo& r);
        armarx::robot_name_service::dto::ArmInfo toIce() const;

    public:
        std::string kinematicChainName;
        TCPInfo hand;
    };

    class RobotInfo
    {
    public:
        void fromIce(const armarx::robot_name_service::dto::RobotInfo& r);
        armarx::robot_name_service::dto::RobotInfo toIce() const;

    public:
        // header
        std::string name;
        armarx::data::PackagePath xmlPackagePath;

        // memory and skills
        armarx::armem::client::MemoryNameSystem memoryNameSystem;
        armarx::skills::manager::dti::SkillManagerInterfacePrx skillManager;

        // kinematic stuff
        std::map<std::string, ArmInfo> armInfos;

        // units
        armarx::RobotUnitInterfacePrx robotUnit;
    };
} // namespace armarx::robot_name_service::core
