#include "SkillDescription.h"

#include <ArmarXCore/core/ice_conversions.h>
#include <ArmarXCore/core/time/ice_conversions.h>

namespace armarx
{
    namespace skills
    {
        provider::dto::SkillDescription
        SkillDescription::toProviderIce() const
        {
            provider::dto::SkillDescription ret;
            ret.parametersType = aron::type::Object::ToAronObjectDTO(parametersType);
            ret.resultType = aron::type::Object::ToAronObjectDTO(resultType);
            ret.description = description;
            ret.skillId = skillId.toProviderIce();
            ret.rootProfileDefaults = aron::data::Dict::ToAronDictDTO(rootProfileDefaults);

            armarx::core::time::toIce(ret.timeout, timeout);
            return ret;
        }

        manager::dto::SkillDescription
        SkillDescription::toManagerIce() const
        {
            manager::dto::SkillDescription ret;
            ret.parametersType = aron::type::Object::ToAronObjectDTO(parametersType);
            ret.resultType = aron::type::Object::ToAronObjectDTO(resultType);
            ret.description = description;
            ret.skillId = skillId.toManagerIce();
            ret.rootProfileDefaults = aron::data::Dict::ToAronDictDTO(rootProfileDefaults);

            armarx::core::time::toIce(ret.timeout, timeout);
            return ret;
        }

        SkillDescription
        SkillDescription::FromIce(const provider::dto::SkillDescription& i,
                                  const std::optional<ProviderID>& pid)
        {
            armarx::core::time::Duration _d;
            armarx::core::time::fromIce(i.timeout, _d);
            return SkillDescription{
                .skillId = SkillID::FromIce(i.skillId, pid),
                .description = i.description,
                .rootProfileDefaults =
                    armarx::aron::data::Dict::FromAronDictDTO(i.rootProfileDefaults),
                .timeout = _d,
                .parametersType = armarx::aron::type::Object::FromAronObjectDTO(i.parametersType),
                .resultType = armarx::aron::type::Object::FromAronObjectDTO(i.resultType)};
        }

        SkillDescription
        SkillDescription::FromIce(const manager::dto::SkillDescription& i)
        {
            armarx::core::time::Duration _d;
            armarx::core::time::fromIce(i.timeout, _d);
            return SkillDescription{
                .skillId = SkillID::FromIce(i.skillId),
                .description = i.description,
                .rootProfileDefaults =
                    armarx::aron::data::Dict::FromAronDictDTO(i.rootProfileDefaults),
                .timeout = _d,
                .parametersType = armarx::aron::type::Object::FromAronObjectDTO(i.parametersType),
                .resultType = armarx::aron::type::Object::FromAronObjectDTO(i.resultType)};
        }
    } // namespace skills
} // namespace armarx
