#pragma once

#include <string>
#include <vector>

#include <ArmarXCore/core/time/DateTime.h>
#include <ArmarXCore/core/time/ice_conversions.h>

#include <RobotAPI/interface/skills/SkillProviderInterface.h>
#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>

#include "SkillID.h"
#include "SkillParameterization.h"

namespace armarx
{
    namespace skills
    {
        struct SkillExecutionID
        {
            SkillID skillId;
            std::string executorName;
            armarx::core::time::DateTime executionStartedTime;

            static const constexpr char* UNKNOWN = "UNKNOWN";
            static const constexpr char* ENTER_SEPARATOR = "[";
            static const constexpr char* EXIT_SEPARATOR = "]";
            static const constexpr char* SEPARATOR = "@";

            bool
            operator==(const SkillExecutionID& other) const
            {
                return this->toString() == other.toString();
            }

            bool
            operator<(const SkillExecutionID& other) const
            {
                return this->executionStartedTime < other.executionStartedTime;
            }

            bool
            operator<=(const SkillExecutionID& other) const
            {
                return this->executionStartedTime <= other.executionStartedTime;
            }

            skills::manager::dto::SkillExecutionID toManagerIce() const;

            skills::provider::dto::SkillExecutionID toProviderIce() const;

            static SkillExecutionID FromIce(const skills::manager::dto::SkillExecutionID&);

            static SkillExecutionID FromIce(const skills::provider::dto::SkillExecutionID&,
                                            const std::optional<skills::ProviderID>& providerName);

            std::string toString() const;
        };

    } // namespace skills
} // namespace armarx
