#include "SkillMemoryGui.h"

#include "gui_utils.h"

namespace armarx::skills::gui
{
    SkillMemoryGUI::SkillMemoryGUI(QTreeWidget* _skillExecutionTreeWidget,
                                   QLayout* _skillExecutionTreeWidgetParentLayout,
                                   QGroupBox* _skillGroupBox,
                                   QSplitter* _skillGroupBoxParentLayout,
                                   QGroupBox* _skillDetailGroupBox,
                                   QSplitter* _skillDetailGroupBoxParentLayout,
                                   QHBoxLayout* _updateWidgetLayout,

                                   QWidget* skillDescription,
                                   QLayout* skillDescriptionParentLayout,

                                   std::shared_ptr<SkillManagerWrapper> _memory)
    {
        Logging::setTag("SkillMemoryGui");

        // parameter check
        ARMARX_CHECK(_skillExecutionTreeWidgetParentLayout);
        ARMARX_CHECK(_skillGroupBoxParentLayout);
        ARMARX_CHECK(_skillDetailGroupBoxParentLayout);
        ARMARX_CHECK(_memory);

        ARMARX_CHECK(_skillExecutionTreeWidget);
        ARMARX_CHECK(_skillGroupBox);
        ARMARX_CHECK(_skillDetailGroupBox);
        ARMARX_CHECK(_updateWidgetLayout);
        ARMARX_CHECK(skillDescription);
        ARMARX_CHECK(skillDescriptionParentLayout);
        ARMARX_CHECK(_memory);

        // setup memory
        this->memory = _memory;

        // Update widget
        this->updateWidgetLayout = _updateWidgetLayout;
        this->updateWidget = new PeriodicUpdateWidget();
        this->updateWidgetLayout->insertWidget(0, updateWidget);


        // replace skillExecutionTreeWidget
        this->skillExecutionTreeWidget = new SkillExecutionTreeWidget(memory);
        armarx::gui::replaceWidget(_skillExecutionTreeWidget,
                                   this->skillExecutionTreeWidget,
                                   _skillExecutionTreeWidgetParentLayout);

        // replace skillGroupBox
        this->skillGroupBox = new SkillGroupBox(memory);

        // replace skillDetailGroupBox
        this->skillDetailGroupBox = new SkillDetailGroupBox(memory);

        armarx::gui::clearSplitter(_skillGroupBoxParentLayout);
        _skillGroupBoxParentLayout->insertWidget(0, this->skillDetailGroupBox);
        _skillGroupBoxParentLayout->insertWidget(0, this->skillGroupBox);

        setupUi();
    }

    void
    SkillMemoryGUI::resetGui()
    {
        ARMARX_ERROR << "Not implemented";
    }

    void
    SkillMemoryGUI::setupUi()
    {
        connectSignals();
    }

    void
    SkillMemoryGUI::connectSignals()
    {
        /*
        // connect update widget to memory update
        connect(this->updateWidget,
                &PeriodicUpdateWidget::update,
                this->memory.get(),
                &SkillManagerWrapper::fetchUpdates);
*/
    }
} // namespace armarx::skills::gui
